@@ -3,9 +3,9 @@
 ## Author: Brice Ozenne
 ## Created: maj 30 2018 (15:58) 
 ## Version: 
-## Last-Updated: okt  4 2021 (20:04) 
-##           By: Brice Ozenne
-##     Update #: 494
+## Last-Updated: Nov 22 2022 (11:49) 
+##           By: Thomas Alexander Gerds
+##     Update #: 531
 ##----------------------------------------------------------------------
 ## 
 ### Commentary: 
@@ -83,6 +83,8 @@ transformIID <- function(estimate, iid, type){
     ## Reference for the formula
     ## Beyersmann, Jan and Allignol, Arthur and Schumacher, Martin. Competing Risks and Multistate Models with R.
     ## Use R! Springer. 2012.
+    # estimate has different time points in columns
+    # and different covariate values in rows
     if(type == "none"){
         ## no change
         return(iid)
@@ -90,21 +92,26 @@ transformIID <- function(estimate, iid, type){
         newiid <- aperm(iid, c(3,2,1))
         if(type == "log"){
             ## formula 4.10 p 58 (Beyersmann et al. 2012)
-            newiid <- sliceScale_cpp(newiid, M = estimate)
+            ## newiid <- sliceScale_cpp(newiid, M = estimate)
+            newiid <- sweep(newiid,MARGIN = 1:2,FUN = "/",STATS = estimate)
         }else if(type == "loglog"){
             ## formula 4.16 p 59 (Beyersmann et al. 2012)
-            newiid <- sliceScale_cpp(newiid, M = - estimate * log(estimate) )
+            ## newiid <- sliceScale_cpp(newiid, M = - estimate * log(estimate) )
+            newiid <- sweep(newiid,MARGIN = 1:2,FUN = "/",STATS = - estimate * log(estimate))
         }else if(type == "cloglog"){
-            newiid <- sliceScale_cpp(newiid, M = - (1 - estimate) * log(1-estimate) )
+            ## newiid <- sliceScale_cpp(newiid, M = - (1 - estimate) * log(1-estimate) )
+            newiid <- sweep(newiid,MARGIN = 1:2,FUN = "/",STATS = - (1 - estimate) * log(1-estimate))
             ## formula 4.21 p 62 (Beyersmann et al. 2012)
         }else if(type == "atanh"){
             ## fisher transform: f(x) = 1/2 log(1+x) - 1/2 log(1-x)
             ##                   df(x) = dx/(2+2x) + dx/(2-2x) = dx(1/(1+x)+1/(1-x))/2
             ##                         = dx/(1-x^2)
             ##               Var(f(x)) = Var(x)/(1-x^2)
-            newiid <- sliceScale_cpp(newiid, M = 1 - estimate^2 )
+            ## newiid <- sliceScale_cpp(newiid, M = 1 - estimate^2 )
+            newiid <- sweep(newiid,MARGIN = 1:2,FUN = "/",STATS = 1 - estimate^2)
         }else if(type == "atanh2"){
-            newiid <- sliceScale_cpp(2*newiid, M = 1 - 4*(estimate-1/2)^2 )
+            ## newiid <- sliceScale_cpp(2*newiid, M = 1 - 4*(estimate-1/2)^2 )
+            newiid <- sweep(2*newiid,MARGIN = 1:2,FUN = "/",STATS = 1 - 4*(estimate-1/2)^2 )
         }
         newiid <- aperm(newiid, c(3,2,1))
     }
@@ -212,6 +219,7 @@ transformT <- function(estimate, se, null, type, alternative){
     }else if(type == "atanh2"){
         statistic <- ( atanh(2*(estimate - null)) )/se
     }
+    statistic[estimate==null] <- 0 ## deal with 0 estimate with 0 variance
 
     return(statistic)
 }
@@ -307,7 +315,7 @@ transformCIBP <- function(estimate, se, iid, null,
     }
     if(!is.na(seed)){set.seed(seed)}
     alternative <- match.arg(alternative, choices = c("two.sided","less","greater"))
-    
+
     ## ** transformation
     ## standard error
     se <- transformSE(estimate = estimate,
@@ -337,8 +345,8 @@ transformCIBP <- function(estimate, se, iid, null,
             n.sample <- dim(iid)[1]
             n.time <- dim(iid)[2]
 
-            ## times with 0 variance (to be removed in further calculaltion as they introduce singularities)
-            index.keep <- which(colSums(abs(se)>1e-12, na.rm = TRUE)>0)[1]:NCOL(se)
+            ## times with 0 variance (to be removed in further calculation as they introduce singularities)
+            index.keep <- which(colSums(abs(se)>1e-12, na.rm = TRUE)>0)[1]:utils::tail(which(colSums(abs(se)>1e-12, na.rm = TRUE)>0),1)
             iid.norm <- array(NA, dim = c(length(index.keep), dim(iid)[1], dim(iid)[3]))
             for(iC in 1:n.contrast){ ## iC <- 1
                 if(length(index.keep)==1){
@@ -397,6 +405,7 @@ transformCIBP <- function(estimate, se, iid, null,
                                                type = type,
                                                min.value = min.value,
                                                max.value = max.value)
+
         if(p.value){
             if(alternative == "two.sided"){
                 if(is.null(df)){
@@ -442,7 +451,8 @@ transformCIBP <- function(estimate, se, iid, null,
             if(method.band == "maxT-integration"){
                 if(band == 1){
                     for(iC in 1:n.contrast){ ## iC <- 1
-                        resQ <- mvtnorm::qmvnorm(p = conf.level, mean = rep(0, n.test),
+
+                        resQ <- mvtnorm::qmvnorm(p = conf.level, mean = rep(0, NCOL(rho[[iC]])),
                                                  sigma = rho[[iC]], tail = switch(alternative,
                                                                                   "two.sided" = "both.tails",
                                                                                   "less" = "upper.tail", ## 'upper.tail' gives x with P[X > x] = p = P[x < X < Inf]
@@ -459,7 +469,10 @@ transformCIBP <- function(estimate, se, iid, null,
                         }
                     }
                 }else if(band == 2){
-                    resQ <- mvtnorm::qmvnorm(p = conf.level, mean = rep(0, n.test),
+                    if(any(is.na(rho)) && any(se==0)){
+                        rho <- rho[se>0,se>0,drop=FALSE]
+                    }
+                    resQ <- mvtnorm::qmvnorm(p = conf.level, mean = rep(0, NCOL(rho)),
                                              cor = rho, tail = switch(alternative,
                                                                       "two.sided" = "both.tails",
                                                                       "less" = "upper.tail", ## 'upper.tail' gives x with P[X > x] = p = P[x < X < Inf]
@@ -517,9 +530,10 @@ transformCIBP <- function(estimate, se, iid, null,
         }
 
         if(p.value){
+            out$adj.p.value <- matrix(NA, nrow = n.contrast, ncol = NCOL(estimate))
+
             if(method.band %in% p.adjust.methods){
                 if(band == 1){
-                    out$adj.p.value <- matrix(NA, nrow = n.contrast, ncol = NCOL(estimate))
                     for(iC in 1:NROW(out[["p.value"]])){
                         out[["adj.p.value"]][iC,] <- stats::p.adjust(out[["p.value"]][iC,], method = method.band)
                     }
@@ -527,52 +541,66 @@ transformCIBP <- function(estimate, se, iid, null,
                     out[["adj.p.value"]] <- stats::p.adjust(out[["p.value"]], method = method.band)
                 }
             }else if(method.band == "maxT-integration"){
-                out$adj.p.value <- matrix(NA, nrow = n.contrast, ncol = NCOL(estimate))
 
                 if(band == 1){
                     for(iC in 1:n.contrast){ ## iC <- 2
-                        out$adj.p.value[iC,] <- sapply(statistic[iC,], function(iT){
+                        iN.test <- NCOL(rho[[iC]])
+
+                        out$adj.p.value[iC,index.keep] <-  sapply(statistic[iC,index.keep], function(iT){ ## iT
                             if(alternative=="two.sided"){
-                                return(1-mvtnorm::pmvnorm(lower=rep(-abs(iT),n.test), upper=rep(abs(iT),n.test),
-                                                          mean=rep(0, n.test), sigma = rho[[iC]]))
+                                return(1-mvtnorm::pmvnorm(lower=rep(-abs(iT), iN.test), upper=rep(abs(iT), iN.test),
+                                                          mean=rep(0, iN.test), sigma = rho[[iC]]))
                             }else if(alternative=="greater"){
-                                return(1-mvtnorm::pmvnorm(lower=rep(-Inf,n.test), upper=rep(iT,n.test),
-                                                          mean=rep(0, n.test), sigma = rho[[iC]]))
+                                return(1-mvtnorm::pmvnorm(lower=rep(-Inf, iN.test), upper=rep(iT, iN.test),
+                                                          mean=rep(0, iN.test), sigma = rho[[iC]]))
                             }else if(alternative=="less"){
-                                return(1-mvtnorm::pmvnorm(lower=rep(iT,n.test), upper=rep(Inf,n.test),
-                                                          mean=rep(0, n.test), sigma = rho[[iC]]))
+                                return(1-mvtnorm::pmvnorm(lower=rep(iT, iN.test), upper=rep(Inf, iN.test),
+                                                          mean=rep(0, iN.test), sigma = rho[[iC]]))
                             }
                         })
+
+                        if(length(index.keep)>0 && all(out$p.value[iC,-index.keep]==1)){
+                            out$adj.p.value[iC,-index.keep] <- out$p.value[iC,-index.keep]
+                        }
                     }
                 }else if(band == 2){
                     for(iC in 1:n.contrast){ ## iC <- 2
-                        out$adj.p.value[iC,] <- sapply(statistic[iC,], function(iT){
+                        iN.test <- NCOL(rho[[iC]])
+
+                        out$adj.p.value[iC,index.keep] <- sapply(statistic[iC,index.keep], function(iT){
                             if(alternative=="two.sided"){
-                                return(1-mvtnorm::pmvnorm(lower=rep(-abs(iT),n.test), upper=rep(abs(iT),n.test),
-                                                          mean=rep(0, n.test), sigma = rho))
+                                return(1-mvtnorm::pmvnorm(lower=rep(-abs(iT), iN.test), upper=rep(abs(iT), iN.test),
+                                                          mean=rep(0, iN.test), sigma = rho))
                             }else if(alternative=="greater"){
-                                return(1-mvtnorm::pmvnorm(lower=rep(-Inf,n.test), upper=rep(iT,n.test),
-                                                          mean=rep(0, n.test), sigma = rho))
+                                return(1-mvtnorm::pmvnorm(lower=rep(-Inf, iN.test), upper=rep(iT,n.test),
+                                                          mean=rep(0, iN.test), sigma = rho))
                             }else if(alternative=="less"){
-                                return(1-mvtnorm::pmvnorm(lower=rep(iT,n.test), upper=rep(Inf,n.test),
-                                                          mean=rep(0, n.test), sigma = rho))
+                                return(1-mvtnorm::pmvnorm(lower=rep(iT, iN.test), upper=rep(Inf, iN.test),
+                                                          mean=rep(0, iN.test), sigma = rho))
                             }
                         })
+
+                        if(length(index.keep)>0 && all(out$p.value[iC,-index.keep]==1)){
+                            out$adj.p.value[iC,-index.keep] <- out$p.value[iC,-index.keep]
+                        }
                     }
                 }
             }else if(method.band == "maxT-simulation"){
-                out$adj.p.value <- pProcess_cpp(nSample = n.sample,
-                                                nContrast = n.contrast,
-                                                nTime = n.time,
-                                                nSim = n.sim,
-                                                value = statistic,
-                                                iid =  iid.norm,
-                                                alternative = switch(alternative,
-                                                                     "two.sided" = 3,
-                                                                     "greater" = 2,
-                                                                     "less" = 1),
-                                                global = (band == 2)                             
-                                                )
+                out$adj.p.value[,index.keep] <- pProcess_cpp(nSample = n.sample,
+                                                             nContrast = n.contrast,
+                                                             nTime = length(index.keep),
+                                                             nSim = n.sim,
+                                                             value = statistic[,index.keep,drop=FALSE],
+                                                             iid =  iid.norm,
+                                                             alternative = switch(alternative,
+                                                                                  "two.sided" = 3,
+                                                                                  "greater" = 2,
+                                                                                  "less" = 1),
+                                                             global = (band == 2)                             
+                                                             )
+                if(length(index.keep)>0 && all(out$p.value[,-index.keep]==1)){
+                    out$adj.p.value[,-index.keep] <- out$p.value[,-index.keep]
+                }
             }
         }
 
